@@ -22,19 +22,29 @@
 #include <stdio.h>
 #include <sysexits.h>
 #include <signal.h>
-#include <stdarg.h>
 #include <npapi.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #include <sys/wait.h>
 #include <errno.h>
-#include <sys/time.h>
 #include <X11/X.h>
-#include <X11/Xos.h>
 #include <X11/Xutil.h>
-#include <X11/Xatom.h>
 
 #include "mozplugger.h"
 
+#define MAX_CONTROLS_WIDTH           300
+#define MAX_CONTROLS_HEIGHT          100
+
+#define DEFAULT_CONTROLS_WIDTH        60
+#define DEFAULT_CONTROLS_HEIGHT       20
+
+#define MIN_CONTROLS_WIDTH            48
+#define MIN_CONTROLS_HEIGHT           16
+
+#define WINDOW_BORDER_WIDTH           1
+
+#define BUTTON_DEPTH                  2
+
 static Display *dpy=0;
 static Window topLevel;
 static int buttonsize=10;
@@ -44,6 +54,8 @@ static int buttonsize=10;
 #define STATE_STOP 3
 
 static int state=STATE_STOP;
+static int buttonDown = -1;
+
 static pid_t pid=-1;
 
 static int repeats = 0;
@@ -54,35 +66,111 @@ static int flags;
 
 static GC gc_white;
 static GC gc_black;
-static GC gc_gray;
+static GC gc_onColor;
+static GC gc_offColor;
 
 static int bx;
-static int by;
 
 static int smallControls=0;
 
-/******************************************************************************
- * Given x,y co-ordinates with the reference framework of a single fixed size
- * button, translate and scale to the actual x,y point within the controller
- * widget on the screen. This function relies on the gloabl variables bx, by
- * and buttonsize being set before calling the function.
+/******************************************************************************/
+/**
+ * @brief scale
+ *
+ * Given v when buttonsize is 16 scale to v if buttonsize was buttonsize,
+ * make sure we round up or down correctly.
+ *
+ * @param[in] v The value to scale
+ *
+ * @return The scaled value
+ *
  *****************************************************************************/
-static XPoint coord(int x, int y)
+static int scale(int v)
+{
+     return (v * buttonsize +8) / 16;
+}
+
+/******************************************************************************/
+/**
+ * @brief coordAdd
+ *
+ * Given origin point and an x and y translation, calculate new point
+ *
+ * @param[in] origin The origin
+ * @param[in] x The X translation 
+ * @param[in] y The Y translation
+ *
+ * @return The co-ordinates as a XPoint structure
+ *
+ *****************************************************************************/
+static XPoint coordAdd(const XPoint * const origin, int x, int y)
 {
      XPoint ret;
-     ret.x=(x+1) * buttonsize / 16 + bx;
-     ret.y=y * buttonsize / 16 + by;
+     ret.x = origin->x + x;
+     ret.y = origin->y + y;
      return ret;
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief draw 3D rectangle
+ *
+ * Draw a rectangle, if pressed is false, give the rectangle shadow to give 
+ * impress of a raised button
+ *
+ * @param[in] x       X-coordinate
+ * @param[in] y       Y-coordinate
+ * @param[in] w       width
+ * @param[in] h       height
+ * @param[in] d       depth
+ * @param[in] c       The color of the button
+ * @param[in] pressed If the button is pressed
+ *
+ *****************************************************************************/
+static void draw3dRectangle(const int x, const int y, const unsigned w, 
+                            const unsigned h, const GC c, const int d, 
+                            const int pressed)
+{
+     if(pressed)
+     {    
+          XFillRectangle(dpy, topLevel, c, x, y, w, h);
+          /* erase left bit of previous not pressed button */
+          XFillRectangle(dpy, topLevel, gc_white, x-d, y-d, d, h+d);
+          /* erase top bit of previous not pressed button */
+          XFillRectangle(dpy, topLevel, gc_white, x, y-d, w, d);
+     }
+     else
+     {  
+          int i;
+
+          XFillRectangle(dpy, topLevel, c, x-d, y-d, w, h);
+
+          /* Draw shadow */
+          for(i=BUTTON_DEPTH; i > 0; i--)
+          {
+               XDrawLine(dpy, topLevel, gc_black, x+w-i, y+1-i, x+w-i, y+h-i);
+               XDrawLine(dpy, topLevel, gc_black, x+1-i, y+h-i, x+w-i, y+h-i);
+          }
+     }
+}
+
+/******************************************************************************/
+/**
+ * @brief redraw
+ *
  * Redraw the three control buttons
+ *
  *****************************************************************************/
 static void redraw(void)
 {
      static int old_buttonsize = -1;
      XWindowAttributes attr;
      XPoint points[6];
+     XPoint base;
+     int x, y;
+     const int d = BUTTON_DEPTH;
+     unsigned w, h;
+     GC c;
 
      XGetWindowAttributes(dpy, topLevel, &attr);
      buttonsize = attr.width/3;
@@ -96,47 +184,105 @@ static void redraw(void)
      }
     
 
-     bx=(attr.width - buttonsize*3)/2;
-     by=(attr.height - buttonsize)/2;
+     base.x = (attr.width - buttonsize*3)/2;
+     base.y = (attr.height - buttonsize)/2;
 
-     /* play */
-     points[0]=coord(4,2);
-     points[1]=coord(5,2);
-     points[2]=coord(10,7);
-     points[3]=coord(10,8);
-     points[4]=coord(5,13);
-     points[5]=coord(4,13);
+     /* Set global variable, required by button press event */
+     bx = base.x;
 
-     XFillPolygon(dpy,
-		  topLevel,
-		  state==STATE_PLAY ? gc_gray : gc_black,
-		  points, 6,
+     /***** play ******/
+
+     {
+         int s = (buttonDown != 0) ? d : 0;
+
+         points[0] = coordAdd(&base,       scale(5)-s,  scale(2)-s);
+         points[1] = coordAdd(&points[0],  scale(2),    0);
+         points[2] = coordAdd(&points[1],  scale(5),    scale(5));
+
+         points[5] = coordAdd(&base,       scale(5)-s, scale(13)-s);
+         points[4] = coordAdd(&points[5],  scale(2),   0);
+         points[3] = coordAdd(&points[4],  scale(5),  -scale(5));
+     }
+
+     c = (state==STATE_PLAY) ? gc_onColor : gc_offColor;
+
+     if(buttonDown != 0)
+     {
+          int i;
+
+          XFillPolygon(dpy, topLevel, c, points, 6,
 		  Convex, CoordModeOrigin);
 
-     /* pause */
-     XFillRectangle(dpy, topLevel, state==STATE_PAUSE ? gc_gray : gc_black,
-		    bx + buttonsize + buttonsize*3/16,
-		    by + buttonsize * 2/16,
-		    (unsigned) (buttonsize * 3/16),
-		    (unsigned) (buttonsize * 11/16));
+          points[2].y++;
+          points[5].x++;
 
-     XFillRectangle(dpy, topLevel, state==STATE_PAUSE ? gc_gray : gc_black,
-		    bx + buttonsize + buttonsize*9/16,
-		    by + buttonsize * 2/16,
-		    (unsigned) (buttonsize * 3/16),
-		    (unsigned) (buttonsize * 11/16));
+          /* Draw the shadow */
+          for(i = BUTTON_DEPTH; i > 0; i--)
+          {
+              XDrawLines(dpy, topLevel, gc_black, &points[2], 4, CoordModeOrigin);
 
-     /* stop */
-     XFillRectangle(dpy, topLevel, state==STATE_STOP ? gc_gray : gc_black,
-		    bx+buttonsize*2 + buttonsize*3/16,
-		    by+buttonsize * 3/16,
-		    (unsigned) (buttonsize * 9/16),
-		    (unsigned) (buttonsize * 9/16));
+              XDrawLine(dpy, topLevel, gc_black, points[3].x-1, points[3].y, 
+                                                 points[4].x-1, points[4].y);
+
+              points[2].x++; points[2].y++;
+              points[3].x++; points[3].y++;
+              points[4].x++; points[4].y++;
+              points[5].x++; points[5].y++;
+          }
+     }
+     else
+     {   
+          XFillPolygon(dpy, topLevel, c, points, 6,
+		  Convex, CoordModeOrigin);
+
+          x = points[0].x;
+          y = points[0].y;
+          w = points[2].x - points[0].x;
+          h = points[4].y - points[0].y;
+
+          /* erase left bit of previous not pressed button */
+          XFillRectangle(dpy, topLevel, gc_white, x-d, y-d, d, h+d);
+          /* erase top bit of previous not pressed button */
+          XFillRectangle(dpy, topLevel, gc_white, x, y-d, w, d);
+     }
+
+     /***** pause *****/
+     base.x += buttonsize;
+     y = base.y + scale(2);
+     w = (unsigned) scale(3) + 1;
+     h = (unsigned) scale(11) + 1;
+     c = (state==STATE_PAUSE) ? gc_onColor : gc_offColor;
+
+     draw3dRectangle(base.x + scale(3),
+                     y, w, h, c, d, buttonDown == 1);
+
+     draw3dRectangle(base.x + scale(9),
+                     y, w, h, c, d, buttonDown == 1);
+
+     /***** stop *****/
+     base.x += buttonsize;
+     w = (unsigned) scale(9);
+
+     draw3dRectangle(base.x + scale(3) + 1,
+                     base.y + scale(3) + 1,
+                     w, w,
+                     (state==STATE_STOP) ? gc_onColor : gc_offColor,
+                     d,
+                     buttonDown == 2);
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Get environment variable as integer
+ *
  * Get the value of environment variable 'var' and convert to an integer value
  * - use 'def' if enviroment variable does not exist
+ *
+ * @param[in] var The name of the variable
+ * @param[in] def The default value to use
+ *
+ * @return The integer value
+ *
  *****************************************************************************/
 static int igetenv(char *var, int def)
 {
@@ -145,8 +291,14 @@ static int igetenv(char *var, int def)
      return atoi(tmp);
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Play
+ *
  * Called when the user clicks on the play button
+ *
+ * @param[in] command The command to execute
+ *
  *****************************************************************************/
 static void my_play(char * command)
 {
@@ -206,8 +358,14 @@ static void my_play(char * command)
      }
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Pause
+ *
  * Called when the user clicks on the pause button
+ *
+ * @param[in] command The command to execute (not used)
+ *
  *****************************************************************************/
 static void my_pause(char * command)
 {
@@ -222,8 +380,12 @@ static void my_pause(char * command)
      }
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Wrapper for kill
+ *
  * Terminate the controller application
+ *
  *****************************************************************************/
 static void low_die(void)
 {
@@ -231,8 +393,14 @@ static void low_die(void)
      _exit(0);
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Stop
+ *
  * Called when the user clicks on the stop button
+ *
+ * @param[in] command The command to execute (not used)
+ *
  *****************************************************************************/
 static void my_stop(char * command)
 {
@@ -245,9 +413,18 @@ static void my_stop(char * command)
      }
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Wrapper for kill
+ *
  * Callback function passed to X for when error occurs. This terminates the 
  * controlled application
+ *
+ * @param[in] dpy The display pointer (not used)
+ * @param[in] ev  The X event         (not used) 
+ *
+ * @return Always returns zero
+ *
  *****************************************************************************/
 static int die(Display *dpy, XErrorEvent *ev) 
 { 
@@ -255,9 +432,17 @@ static int die(Display *dpy, XErrorEvent *ev)
      return 0; 
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Wrapper for kill
+ *
  * Callback function passed to X for when error occurs. This terminates the 
  * controlled application
+ *
+ * @param[in] dpy The display pointer (not used)
+ *
+ * @return Always returns zero
+ *
  *****************************************************************************/
 static int die2(Display *dpy) 
 { 
@@ -265,17 +450,33 @@ static int die2(Display *dpy)
      return 0; 
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Wrapper for kill
+ *
  * Callback function attached as a signal handler. This terminates the 
  * controlled application
+ *
+ * @param[in] sig The signal (not used)
+ *
  *****************************************************************************/
 static void sigdie(int sig) 
 { 
      low_die(); 
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Callback filter for X events
+ *
  * Defines the X events that we wish to see. Returns true for everything. 
+ *
+ * @param[in] dpy The display pointer
+ * @param[in] ev  Pointer to the event data
+ * @param[in] arg Any associated event data
+ *
+ * @return Always returns true
+ *
  *****************************************************************************/
 static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 {
@@ -283,22 +484,32 @@ static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 }
 
 
-/******************************************************************************
+/******************************************************************************/
+/** 
+ * @brief Normalise window co-ordinates
+ *
  * Given initial width & height of the window containing the buttons 
  * limit the width and aspect ratio and place in the center of the 
  * window (calculate x & y)
+ *
+ * @param[in,out] width  The width before and after
+ * @param[in,out] height The height before and after
+ * @param[in,out] x      The x co-ordinate before and after
+ * @param[in,out] y      The y co-ordinate before and after
+ *
  ******************************************************************************/
 static void normalise_window_coords(unsigned * width, unsigned * height,
                                     int * x, int * y)
 {
     unsigned w = *width;
     unsigned h = *height;
-
+    const unsigned target_aspect = DEFAULT_CONTROLS_WIDTH /
+                                               DEFAULT_CONTROLS_HEIGHT;
     if(smallControls)
     {
         *y = *x = 0;
-	*width = 48;
-        *height = 16;
+	*width  = MIN_CONTROLS_WIDTH;
+        *height = MIN_CONTROLS_HEIGHT;
         return;
     }
 
@@ -308,15 +519,15 @@ static void normalise_window_coords(unsigned * width, unsigned * height,
         return;
     }
 
-    if((w > 300) & !(flags & H_MAXASPECT))
+    if((w > MAX_CONTROLS_WIDTH) & !(flags & H_MAXASPECT))
     {
-        w = 300;
+        w = MAX_CONTROLS_WIDTH;
     }
 
-    /* Keep the controls as close to 3:1 ratio as possible */
-    if(h > w/3)
+    /* Keep the controls as close to default ratio as possible */
+    if(h > w / target_aspect)
     {
-        h = w/3;
+        h = w / target_aspect;
     }
 
     *x = (int) (*width - w)/2;
@@ -326,9 +537,13 @@ static void normalise_window_coords(unsigned * width, unsigned * height,
     *width = w;
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Check incoming pipe 
+ *
  * Check to see if new window size information has arrived from plugin and if
  * so resize the controls accordingly
+ *
  ******************************************************************************/
 static void check_pipe_fd_events(void)
 {
@@ -357,10 +572,9 @@ static void check_pipe_fd_events(void)
 	  return;
      }
 
-     /* Adjust the width & height to compensate for the window border of 
-      * 1 pixel */
-     w = (unsigned) wintmp.width - 2;
-     h = (unsigned) wintmp.height - 2;
+     /* Adjust the width & height to compensate for the window border width */
+     w = (unsigned) wintmp.width -  2 * WINDOW_BORDER_WIDTH;
+     h = (unsigned) wintmp.height - 2 * WINDOW_BORDER_WIDTH;
 
      normalise_window_coords(&w, &h, &x, &y);
 
@@ -369,10 +583,14 @@ static void check_pipe_fd_events(void)
      XMoveResizeWindow(dpy, topLevel, x, y, w, h);
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief Exit early
+ *
  * Called if we exit main() early because of invalid passed argument into
  * the main function. print to both debug and stderr (just in case someone
  * called mozplugger-controller directly)
+ *
  *****************************************************************************/
 static void exitEarly(void)
 {
@@ -384,7 +602,10 @@ static void exitEarly(void)
      exit(1);
 }
 
-/******************************************************************************
+/******************************************************************************/
+/**
+ * @brief main
+ *
  * mozplugger-controller takes two arguments. The first is a shell command line
  * to run when the 'play' button is pressed (or if autostart is set). The second
  * parameter is optional and is the ID of the parent window into which to draw
@@ -393,12 +614,20 @@ static void exitEarly(void)
  * If the command line contains the string "$window", then no controls are 
  * drawn.
  *
+ * @param[in] argc The number of arguments
+ * @param[in] argv Array of the arguments
+ *
+ * @return Never returns unless the application exits
+ *
  ******************************************************************************/
 int main(int argc, char **argv)
 {
      int old_state;
-     unsigned int width = 60;       /* Set defaults, may be changed later */
-     unsigned int height = 20;
+
+     /* Set defaults, may be changed later */
+     unsigned int width =  DEFAULT_CONTROLS_WIDTH; 
+     unsigned int height = DEFAULT_CONTROLS_HEIGHT;
+
      Window parentWid;
      unsigned long temp = 0;
      int x, y ;
@@ -422,8 +651,8 @@ int main(int argc, char **argv)
 	    &repeatsResetVal,
 	    &pipe_fd,
 	    &temp,
-	    (int *)&x,
-	    (int *)&y,
+	    (int *)&x,  /* Not needed and not used */
+	    (int *)&y,  /* Not needed and not used */
 	    (int *)&width,
 	    (int *)&height);
 
@@ -462,10 +691,10 @@ int main(int argc, char **argv)
 	  smallControls=1;
      }
 
-     /* Adjust the width & height to compensate for the window border of 
-      * 1 pixel */
-     width -= 2;
-     height -= 2;
+     /* Adjust the width & height to compensate for the window border
+      * width */
+     width  -= 2 * WINDOW_BORDER_WIDTH;
+     height -= 2 * WINDOW_BORDER_WIDTH;
 
      /* x, y co-ords of the parent is of no interest, we need to know
       * the x, y relative to the parent. */
@@ -475,21 +704,23 @@ int main(int argc, char **argv)
 
      wmHints.x=0;
      wmHints.y=0;
-     wmHints.min_width = 48;
-     wmHints.min_height = 16;
-     wmHints.base_width = 60;
-     wmHints.base_height = 20;
+     wmHints.min_width  = MIN_CONTROLS_WIDTH;
+     wmHints.min_height = MIN_CONTROLS_HEIGHT;
+     wmHints.base_width  = DEFAULT_CONTROLS_WIDTH;
+     wmHints.base_height = DEFAULT_CONTROLS_HEIGHT;
      wmHints.flags=PPosition | USPosition | PMinSize;
 
      attr.border_pixel = 0; 
      attr.background_pixel = WhitePixelOfScreen(DefaultScreenOfDisplay(dpy));
-     attr.event_mask = ExposureMask | ButtonPressMask;
+     attr.event_mask = ExposureMask | ButtonPressMask | ButtonReleaseMask;
      attr.override_redirect=0;
 
      topLevel = XCreateWindow(dpy,
 			      parentWid,
 			      x, y,
-			      width, height, 1, CopyFromParent,
+			      width, height, 
+                              WINDOW_BORDER_WIDTH, 
+                              CopyFromParent,
 			      InputOutput, (Visual *) CopyFromParent,
 			      (unsigned long)(CWBorderPixel|
 			       CWEventMask|
@@ -508,7 +739,7 @@ int main(int argc, char **argv)
      gc_white=XCreateGC(dpy,topLevel,0,0);
      XSetForeground(dpy,gc_white,WhitePixel(dpy,DefaultScreen(dpy)));
 
-     gc_gray=XCreateGC(dpy,topLevel,0,0);
+     gc_onColor=XCreateGC(dpy,topLevel,0,0);
      colour.red=0x0000;
      colour.green=0xa000;
      colour.blue=0x0000;
@@ -516,7 +747,17 @@ int main(int argc, char **argv)
      colour.flags=0;
 
      XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &colour);
-     XSetForeground(dpy,gc_gray,colour.pixel);
+     XSetForeground(dpy,gc_onColor,colour.pixel);
+
+     gc_offColor=XCreateGC(dpy,topLevel,0,0);
+     colour.red=0x8000;
+     colour.green=0x8000;
+     colour.blue=0x8000;
+     colour.pixel=0;
+     colour.flags=0;
+
+     XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &colour);
+     XSetForeground(dpy,gc_offColor,colour.pixel);
 
      XSetWMNormalHints(dpy, topLevel, &wmHints);
 
@@ -590,8 +831,11 @@ int main(int argc, char **argv)
 	       case ButtonPress:
 		    if(ev.xbutton.button == 1)
 		    {
-			 int button = (ev.xbutton.x - bx) / buttonsize;
-			 switch(button)
+			 buttonDown = (ev.xbutton.x - bx) / buttonsize;
+                         redraw();  /* do first gives quicker visual feedback */
+                         XSync(dpy, False);
+
+                         switch(buttonDown)
 			 {
 			 case 0: /* play */ my_play(command); break;
 			 case 1: /* pause*/ my_pause(command);break;
@@ -599,7 +843,15 @@ int main(int argc, char **argv)
 			 }
 		    }
 		    break;
-	  
+	 
+               case ButtonRelease:
+                    if(buttonDown != -1)
+                    {   
+                        buttonDown = -1;
+                        redraw();
+                    }     
+                    break;
+
 	       case Expose:
 		    if(ev.xexpose.count) break;
 	       case ResizeRequest:
