@@ -36,16 +36,9 @@
 
 #include "mozplugger.h"
 
-static Display * display = 0;
-static int pipe_fd;
-static int flags;
-static int repeats;
-static char *winname;
-static char *command;
-static char *file;
-
-static XWindowAttributes wattr;
-
+/*****************************************************************************
+ * Type declarations
+ *****************************************************************************/
 static struct
 {
      int    mapped;
@@ -65,11 +58,30 @@ static struct
     int     height;
 } parentDetails;
 
+
+/*****************************************************************************
+ * Global variables
+ *****************************************************************************/
+static Display * display = 0;
+static int pipe_fd;
+static int flags;
+static int repeats;
+static char *winname;
+static char *command;
+static char *file;
+
+static XWindowAttributes wattr;
+
 static Atom swallowMutex;
 static int  swallowMutexTaken;
 static Atom windowOwnerMark;
 
-#define WINDOW (parentDetails.window)
+static int xaspect;
+static int yaspect;
+
+#define MAX_IGNORED 3000
+static unsigned int ignore_window_count = 0;
+static Window ignored[MAX_IGNORED];
 
 /*****************************************************************************/
 /**
@@ -250,8 +262,8 @@ static void setSwallowMutexOwner(uint32 hostId, uint32 pid)
 static void takeSwallowMutex(void)
 {
     int countDown;
-    uint32 ourPid = (uint32)getpid();
-    uint32 ourHostId = getHostId();
+    const uint32 ourPid = (uint32)getpid();
+    const uint32 ourHostId = getHostId();
     uint32 otherPid;
     uint32 otherHostId;
     uint32 prevOtherPid = 0;
@@ -314,14 +326,48 @@ static void takeSwallowMutex(void)
             }
 	
             usleep(250000);        /* 250ms */
- 	}  
+ 	}
+
         /* else no one has semaphore, timeout, or owner is dead -
            Set us as the owner, but we need to check if we weren't 
-           beaten to it so once more around the loop */
+           beaten to it so once more around the loop.
+           Note even doing the recheck does not work in 100% of all
+           cases due to task switching occuring at just the wrong moment
+           see Mozdev bug 20088 - the fix is done use the stillHaveMutex
+           function */
+
         setSwallowMutexOwner(ourHostId, ourPid);
     }
 }
 
+/*****************************************************************************/
+/**
+ * @brief Check if we still have the mutex semaphore
+ *
+ * Return true if we still have the Mutex semaphore
+ *
+ *****************************************************************************/
+static int stillHaveMutex(void)
+{
+     uint32 otherPid;
+     uint32 otherHostId;
+
+     if(getSwallowMutexOwner(&otherHostId, &otherPid))
+     {
+          const uint32 ourHostId = getHostId();
+          if( otherHostId == ourHostId)    
+          {
+               const uint32 ourPid = (uint32)getpid();
+               if(otherPid == ourPid)     
+               {
+                    return 1;
+               }
+          }
+     }
+     D("Race condition detected, semaphore pinched by Pid(%lu)\n", otherPid);
+     return 0;
+}
+ 
 /*****************************************************************************/
 /**
  * @brief Give the mutex semaphore
@@ -397,13 +443,6 @@ static int chkAndMarkVictimWindow(Window w)
      return gotIt;
 }
 
-/*****************************************************************************
- * Window resizing
- *****************************************************************************/
-
-static int xaspect;
-static int yaspect;
-
 /*****************************************************************************/
 /**
  * @brief Calculate scaling factor
@@ -437,9 +476,9 @@ static int gcd(int a, int b)
  *****************************************************************************/
 static int set_aspect(int x, int y)
 {
-     int ox = xaspect;
-     int oy = yaspect;
-     int d = gcd(x, y);
+     const int ox = xaspect;
+     const int oy = yaspect;
+     const int d = gcd(x, y);
      xaspect = x / d;
      yaspect = y / d;
      D("xaspect=%d yaspect=%d\n", xaspect, yaspect);
@@ -572,7 +611,7 @@ static void change_leader(void)
 static int reparent_window(void)
 {
      if (!victimDetails.window) return 0;
-     if (!WINDOW) return 0;
+     if (!parentDetails.window) return 0;
 
      XSelectInput(display, victimDetails.window, StructureNotifyMask); 
      XSync(display, False);
@@ -581,8 +620,8 @@ static int reparent_window(void)
      change_leader();
      
      D("Reparenting window %x into %x\n", (unsigned)victimDetails.window, 
-                                                             (unsigned)WINDOW);
-     XReparentWindow(display, victimDetails.window, WINDOW, 0, 0);
+                                                (unsigned)parentDetails.window);
+     XReparentWindow(display, victimDetails.window, parentDetails.window, 0, 0);
 
      D("reparent_window() done\n");
 
@@ -694,9 +733,6 @@ static char check_window_name(Window w, char *name)
      return 0;
 }
 
-#define MAX_IGNORED 3000
-static unsigned int ignore_window_count = 0;
-static Window ignored[MAX_IGNORED];
 
 /*****************************************************************************
  * @brief initialise recurwivw search for windows
@@ -851,7 +887,7 @@ static int setup_display(void)
 {
      char *displayname;
 
-     if(WINDOW == 0)       /* mozdev bug #18837 */
+     if(parentDetails.window == 0)       /* mozdev bug #18837 */
      {
           D("setup_display() WINDOW is Null - so nothing setup\n");
           return 1;
@@ -868,7 +904,7 @@ static int setup_display(void)
 	  return 1;
      }
 
-     if (!XGetWindowAttributes(display, WINDOW, &wattr))
+     if (!XGetWindowAttributes(display, parentDetails.window, &wattr))
      {
           D("setup_display() failed cannot get window attributes!!\n");
           XCloseDisplay(display);
@@ -877,7 +913,7 @@ static int setup_display(void)
      }
 
      D("display=%x\n", (unsigned) display);
-     D("WINDOW=%x\n", (unsigned) WINDOW);
+     D("WINDOW=%x\n", (unsigned) parentDetails.window);
      D("rootwin=%x\n", (unsigned) wattr.root);
      
      D("setup_display() done\n");
@@ -1063,7 +1099,7 @@ static void check_x_events(void)
 		    break;
 	    
 	       case ReparentNotify:
-		    if (ev.xreparent.parent == WINDOW)
+		    if (ev.xreparent.parent == parentDetails.window)
 		    {
 			 D("REPARENT NOTIFY on victim to the right window\n");
 			 adjust_window_size();
@@ -1094,7 +1130,8 @@ static void check_x_events(void)
                    break;
 	       }
 	  }
-	  else if (WINDOW && ev.xany.window == WINDOW)
+          else if (parentDetails.window 
+                                    && (ev.xany.window == parentDetails.window))
 	  {
                unsigned long mask;
 	       switch (ev.type)
@@ -1159,8 +1196,9 @@ static void check_pipe_fd_events(void)
      static NPWindow wintmp;
      int n;
   
-     Window oldwindow = WINDOW;
-     D("Got pipe_fd data, old parent=%x pipe_fd=%d\n", (unsigned) WINDOW, pipe_fd);
+     Window oldwindow = parentDetails.window;
+     D("Got pipe_fd data, old parent=%x pipe_fd=%d\n", 
+                                                 (unsigned) oldwindow, pipe_fd);
   
      n = read(pipe_fd, ((char *)& wintmp),  sizeof(wintmp));
      if (n < 0)
@@ -1185,13 +1223,13 @@ static void check_pipe_fd_events(void)
      parentDetails.width = (int) wintmp.width;
      parentDetails.height = (int) wintmp.height;
 
-     D("Got pipe_fd data, new parent=%x\n", (unsigned) WINDOW);
+     D("Got pipe_fd data, new parent=%x\n", (unsigned) parentDetails.window);
   
-     if (WINDOW && WINDOW != oldwindow && display)
+     if (parentDetails.window && (parentDetails.window != oldwindow) && display)
      {
           victimDetails.reparented = 0;
 	  XSelectInput(display, oldwindow, 0);
-	  XSelectInput(display, WINDOW, SubstructureRedirectMask);
+	  XSelectInput(display, parentDetails.window, SubstructureRedirectMask);
 	  XSync(display, False);
           reparent_window();
      }
@@ -1396,11 +1434,23 @@ int main(int argc, char **argv)
 	       /* Belt & braces, make sure we give semaphore on error. */
 	       signal(SIGTERM, giveSwallowMutex);
 
-	       takeSwallowMutex(); 
+               /* There is a race condition when taking the semaphore that 
+                * means occasionally we think we have it but we dont
+                * This do-while loop checks for that case - this
+                * is better than putting a wait in the take semaphore
+                * rechecking loop fixes Mozdev bug 20088 */
 
-	       /* Has to be done before forking */
-	       save_initial_windows();
-	       XSelectInput(display, WINDOW, SubstructureRedirectMask);
+               do
+               {
+	           takeSwallowMutex(); 
+
+	           /* Has to be done before forking */
+	           save_initial_windows();
+               }
+               while(!stillHaveMutex());
+
+               XSelectInput(display, parentDetails.window, 
+                                                      SubstructureRedirectMask);
 	       XSelectInput(display, wattr.root, SubstructureNotifyMask);
 	       XSync(display, False);
 	  }
