@@ -33,8 +33,6 @@
 
 #include "mozplugger.h"
 
-#define THIS ((data_t *)(instance->pdata))
-
 #ifndef __GNUC__
 #define __inline
 #endif
@@ -63,6 +61,7 @@ typedef struct data
      char *mimetype;
      char *href;
      char *mms;
+     char *urlFragment;
      char autostart;
      char autostartNotSeen;
      int num_arguments;
@@ -96,6 +95,12 @@ typedef struct
      unsigned int value;
 } flag_t;
 
+typedef struct 
+{
+    char name[SMALL_BUFFER_SIZE];
+    short exists;
+} cacheEntry_t;
+
 /*****************************************************************************
  * Global variables
  *****************************************************************************/
@@ -108,6 +113,9 @@ static handler_t handlers[MAX_NUM_HANDLERS];
 static int browserApiMajorVer;
 static int browserApiMinorVer;
 
+static char staticPool[MAX_STATIC_MEMORY_POOL];
+static int  staticPoolIdx = 0;
+
 /*****************************************************************************/
 /**
  * @brief Wrapper for fork
@@ -117,14 +125,14 @@ static int browserApiMinorVer;
  * Also it closes all those file descriptors inherited from the parent 
  * (except STDIN, STDOUT & STDERR (0,1,2))
  *
- * @param[in] instance The plugin instance
+ * @param[in] THIS     Pointer to data associated with this instance of the plugin
  * @param[in] pipeFd   The file descriptor of the pipe used to communicate with
  *                     the helper app
  *
  * @return The process ID of the child process created
  *
  *****************************************************************************/
-static pid_t my_fork(NPP instance, int pipeFd)
+static pid_t my_fork(data_t * const THIS, int pipeFd)
 {
      pid_t pid;
      sigset_t set;
@@ -221,7 +229,7 @@ static pid_t my_fork(NPP instance, int pipeFd)
 static void my_putenv(char *buffer, int *offset, const char *var, 
                                                  const char *value)
 {
-     int l = strlen(var) + strlen(value) + 2;
+     const int l = strlen(var) + strlen(value) + 2;
      if (*offset + l >= ENV_BUFFER_SIZE)
      {
 	  D("Buffer overflow in putenv(%s=%s)\n", var, value);
@@ -243,12 +251,13 @@ static void my_putenv(char *buffer, int *offset, const char *var,
  * Since Linux uses copy-on-write, best leave memory read-only and once execlp
  * is called, all daughter memory is wiped anyway (except the stack).
  *
- * @param[in] instance Pointer to the plugin instance data
+ * @param[in] THIS     Pointer to the data associated with this instance of the
+ *                     plugin
  * @param[in] file     The url of the embedded object
  * @param[in] fd       The file descriptor of the pipe to the helper application
  *
  *****************************************************************************/
-static void run(NPP instance, const char *file, int fd)
+static void run(data_t * const THIS, const char *file, int fd)
 {
      char buffer[ENV_BUFFER_SIZE];
      char foo[SMALL_BUFFER_SIZE];
@@ -306,17 +315,30 @@ static void run(NPP instance, const char *file, int fd)
      
      my_putenv(buffer, &offset, "file", file);
 
+     if(THIS->urlFragment)
+     {
+          my_putenv(buffer, &offset, "fragment", THIS->urlFragment);
+     }
+
      my_putenv(buffer, &offset, "autostart", autostart ? "1" : "0");
       
      if (THIS->winname)
+     {
 	  my_putenv(buffer, &offset, "winname", THIS->winname);
-     
+     }
+
      if (THIS->displayname)
+     {
 	  my_putenv(buffer, &offset, "DISPLAY", THIS->displayname);
+     }
 
      for (i = 0; i < THIS->num_arguments; i++)
+     {
 	  if (THIS->args[i].value)
+          {
 	       my_putenv(buffer, &offset, THIS->args[i].name, THIS->args[i].value);
+          }
+     }
 
      if(flags & H_CONTROLS)
      {
@@ -405,12 +427,6 @@ static int inpath(char *path, const char *file)
      return 0;
 }
 
-typedef struct 
-{
-    char name[SMALL_BUFFER_SIZE];
-    short exists;
-} cacheEntry_t;
-
 /*****************************************************************************/
 /**
  * @brief Find a file
@@ -431,7 +447,7 @@ static int find(const char *file)
 
      struct stat filestat;
      int i;
-     int exists;
+     int exists = 0;
 
      D("find(%s)\n", file);
 
@@ -451,7 +467,6 @@ static int find(const char *file)
      }
      else
      {
-          char *path;
           char *env_path;
 
           /* Get the environment variable PATH */
@@ -463,9 +478,12 @@ static int find(const char *file)
           else
           {
                /* Duplicate to avoid corrupting Mozilla's PATH */
-               path = strdup(env_path);
-               exists = inpath(path, file);
-               free(path);
+               char * const path = strdup(env_path);
+               if(path)
+               {
+                    exists = inpath(path, file);
+                    free(path);
+               }
           }
      }
 
@@ -507,8 +525,6 @@ static void filter_previous_handler(void)
      }
 }
 
-static char staticPool[MAX_STATIC_MEMORY_POOL];
-static int  staticPoolIdx = 0;
 
 /*****************************************************************************/
 /**
@@ -605,9 +621,11 @@ static char *get_parameter(char *x, const char *flag, const char **c)
 
 /*****************************************************************************/
 /**
- * @brief Check if two strings match
+ * @brief Check if line starts with the keyword
  *
- * Match two words.
+ * Checks if 'line' starts with 'kw' (keyword), if 'line' is longer than 'kw', 
+ * checks that there is not an additional alpha-numeric letter in the 'line'
+ * after the 'kw'.
  *
  * @param[in] line The input text line being parsed
  * @param[in] kw   The keyword to match against
@@ -618,7 +636,9 @@ static char *get_parameter(char *x, const char *flag, const char **c)
 __inline
 static int match_word(const char *line, const char *kw)
 {
-     return !strncasecmp(line, kw, strlen(kw)) && !isalnum(line[strlen(kw)]);
+     const unsigned kwLen = strlen(kw);
+
+     return (strncasecmp(line, kw, kwLen) == 0) && !isalnum(line[kwLen]);
 }
 
 /*****************************************************************************/
@@ -635,7 +655,7 @@ static int match_word(const char *line, const char *kw)
  *****************************************************************************/
 static int parse_flags(char **x, command_t *commandp)
 {
-     static flag_t flags[] = {
+     const static flag_t flags[] = {
 	  { "repeat", 		H_REPEATCOUNT 	},
 	  { "loop", 		H_LOOP 		},
 	  { "stream", 		H_STREAM 	},
@@ -655,7 +675,8 @@ static int parse_flags(char **x, command_t *commandp)
           { "autostart",        H_AUTOSTART     },
 	  { NULL, 		0 		}
      };
-     flag_t *f;
+
+     const flag_t *f;
 
      for (f = flags; f->name; f++)
      {
@@ -1121,6 +1142,34 @@ static int safeURL(const char* url)
      return 1;
 }
 
+/*****************************************************************************/
+/**
+ * @brief Extract fragment from URL
+ *
+ * Extract the 'fragment' from the end of the URL if present. This is passed
+ * via the fragment environment varaible to the helper application
+ *
+ * @param[in] url The URL to parse
+ *
+ * @return Nothing
+ *
+ *****************************************************************************/
+static void parseURL(data_t * const THIS, const char * url)
+{
+     const char * frag = strchr(url, '#');
+     if(frag)
+     {
+          if(THIS->urlFragment)
+          {
+              D("parseURL - replacing previous fragment\n");
+              free(THIS->urlFragment);
+          }
+
+          D("parseURL - fragment '%s' found at end of URL\n", &frag[1]);
+          THIS->urlFragment = strdup(&frag[1]);
+     }
+}
+	
 /*****************************************************************************/
 /**
  * @brief Match URL
@@ -1180,16 +1229,17 @@ static int match_url(const char * matchStr, const char * url)
  *
  * Go through the commands in the config file and find one that fits our needs.
  *
- * @param[in] instance  Pointer to the plugin instance data
- * @param[in] streaming True if streaming, false if not
- * @param[in] c         Pointer to command structure to match against
- * @param[in] url       The url
+ * @param[in] THIS       Pointer to the data associated with this instance of the
+ *                       plugin
+ * @param[in] streamOnly If true select entry sith stream set only
+ * @param[in] c          Pointer to command structure to match against
+ * @param[in] url        The url
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
 __inline
-static int match_command(NPP instance, int streaming, 
+static int match_command(data_t * const THIS, int streamOnly, 
                          command_t *c, const char * url)
 {
 #define MODE_MASK (H_NOEMBED | H_EMBED | H_LINKS)
@@ -1213,9 +1263,9 @@ static int match_command(NPP instance, int streaming,
 	  D("Flag mismatch: loop\n");
 	  return 0;
      }
-     if ((!!streaming) != (!!(c->flags & H_STREAM)))
+     if (streamOnly && !(c->flags & H_STREAM))
      {
-	  D("Flag mismatch: stream\n");
+	  D("Flag mismatch: stream only required\n");
 	  return 0;
      }
 
@@ -1238,14 +1288,15 @@ static int match_command(NPP instance, int streaming,
  *
  * See if mimetype matches.
  *
- * @param[in] instance Pointer to plugin instance data
+ * @param[in] THIS     pointer to data associated with this instance of the 
+ *                     plugin
  * @param[in] m        Mimetype to match against
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
 __inline
-static int match_mime_type(NPP instance, mimetype_t *m)
+static int match_mime_type(data_t * const THIS, mimetype_t *m)
 {
      char mimetype[SMALL_BUFFER_SIZE];
      sscanf(m->type, "%"SMALL_BUFFER_SIZE_STR"[^:]", mimetype);
@@ -1269,17 +1320,17 @@ static int match_mime_type(NPP instance, mimetype_t *m)
  *
  * See if handler matches.
  *
- * @param[in] h         Pointer to handler to match against
- * @param[in] instance  Pointer to plugin instance data
- * @param[in] streaming True if streaming
- * @param[in] url       The URL 
+ * @param[in] h          Pointer to handler to match against
+ * @param[in] THIS       Pointer to data associated with this instance of plugin
+ * @param[in] streamOnly If True select only entry with stream flag
+ * @param[in] url        The URL 
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
 __inline
-static int match_handler(handler_t *h, NPP instance, 
-                         int streaming, const char *url)
+static int match_handler(handler_t *h, data_t * const THIS, 
+                         int streamOnly, const char *url)
 {
      mimetype_t *m;
      command_t *c;
@@ -1291,12 +1342,12 @@ static int match_handler(handler_t *h, NPP instance,
      for (mid = 0; mid < h->num_types; mid++)
      {
 	  m = &h->types[mid];
-	  if (match_mime_type(instance, m))
+	  if (match_mime_type(THIS, m))
 	  {
 	       for (cid = 0; cid < h->num_cmds; cid++)
 	       {
 		    c = &h->cmds[cid];
-		    if (match_command(instance, streaming, c, url))
+		    if (match_command(THIS, streamOnly, c, url))
 		    {
 			 THIS->cmd_flags = c->flags;
 			 THIS->command = c->cmd;
@@ -1315,14 +1366,14 @@ static int match_handler(handler_t *h, NPP instance,
  *
  * Find the appropriate command
  *
- * @param[in] instance  Pointer to plugin instance data
- * @param[in] streaming True if streaming
- * @param[in] url       The URL  
+ * @param[in] THIS        Pointer to plugin instance data
+ * @param[in] streamOnly  If true select only the command with stream flag
+ * @param[in] url         The URL  
  *
  * @return 1(true) if match, else zero otherwise
  *
  *****************************************************************************/
-static int find_command(NPP instance, int streaming, const char * url)
+static int find_command(data_t * const THIS, int streamOnly, const char * url)
 {
      int hid;
 
@@ -1332,7 +1383,7 @@ static int find_command(NPP instance, int streaming, const char * url)
 
      for (hid = 0; hid < num_handlers; hid++)
      {
-	  if (match_handler(&handlers[hid], instance, streaming, url))
+	  if (match_handler(&handlers[hid], THIS, streamOnly, url))
 	  {
 	       D("Command found.\n");
 	       return 1;
@@ -1545,6 +1596,7 @@ NPError NPP_New(NPMIMEType pluginType,
      int autostart_idx = -1;
      int autohref_idx = -1;
      int target_idx = -1;
+     data_t * THIS;
 
      char *url = NULL;
 
@@ -1560,10 +1612,12 @@ NPError NPP_New(NPMIMEType pluginType,
 	  return NPERR_INVALID_INSTANCE_ERROR;
      }
 
-     instance->pdata = NPN_MemAlloc(sizeof(data_t));
-     if (instance->pdata == NULL) 
+     THIS = NPN_MemAlloc(sizeof(data_t));
+     if (THIS == NULL) 
+     {
           return NPERR_OUT_OF_MEMORY_ERROR;
-
+     }
+     instance->pdata = THIS;
 
      memset((void *)THIS, 0, sizeof(data_t));
 
@@ -1760,12 +1814,16 @@ NPError NPP_New(NPMIMEType pluginType,
 NPError NPP_Destroy(NPP instance, NPSavedData** save)
 {
      int e;
+     data_t * THIS;
 
      D("Destroy() - instance=%p\n", instance);
 
      if (!instance)
+     {
 	  return NPERR_INVALID_INSTANCE_ERROR;
+     }
 
+     THIS = instance->pdata;
      if (THIS)
      {
 	  /* Kill the mozplugger-helper process and his sons */
@@ -1787,6 +1845,9 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 	  THIS->href = NULL;
 	  THIS->mms = NULL;
 
+          free(THIS->urlFragment);
+          THIS->urlFragment = NULL;
+
 	  NPN_MemFree(instance->pdata);
 	  instance->pdata = NULL;
      }
@@ -1802,18 +1863,23 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
  *
  * Check that no child is already running before forking one.
  *
- * @param[in] instance Pointer to the plugin instance data
+ * @param[in] THIS     Pointer to the plugin instance data
  * @param[in] fname    The filename of the embedded object
  *
  *****************************************************************************/
 static void new_child(NPP instance, const char* fname)
 {
      int pipe[2];
+     data_t * THIS;
 
      D("NEW_CHILD(%s)\n", fname);
 
      if (!instance || !fname)
+     {
 	  return;
+     }
+
+     THIS = instance->pdata;
 
      if (THIS->pid != -1)
 	  return;
@@ -1828,7 +1894,7 @@ static void new_child(NPP instance, const char* fname)
 	  return;
      }
 
-     if ((THIS->pid = my_fork(instance, pipe[1])) == -1)
+     if ((THIS->pid = my_fork(THIS, pipe[1])) == -1)
      {
 	  NPN_Status(instance, "MozPlugger: Failed to fork helper!");
 	  return;
@@ -1839,7 +1905,7 @@ static void new_child(NPP instance, const char* fname)
 	  close(pipe[0]);
 
 	  D("CHILD RUNNING run() [2]\n");
-	  run(instance, fname, pipe[1]);
+	  run(THIS, fname, pipe[1]);
 
 	  _exit(EX_UNAVAILABLE);
      }
@@ -1873,11 +1939,16 @@ NPError NPP_NewStream(NPP instance,
 		      uint16 *stype)
 {
      char * savedMimetype = NULL;
+     data_t * THIS;
 
      D("NewStream() - instance=%p\n", instance);
 
      if (instance == NULL)
+     {
 	  return NPERR_INVALID_INSTANCE_ERROR;
+     }
+
+     THIS = instance->pdata;
 
      if (THIS->pid != -1)
      {
@@ -1901,6 +1972,9 @@ NPError NPP_NewStream(NPP instance,
 	  stream->url = THIS->href;
      }
 
+     /* Extract from the URL the various additional information */
+     parseURL(THIS, stream->url);
+
      D("Mime type %s\n", type);
      D("Url is %s (seekable=%d)\n", stream->url, seekable);
 
@@ -1920,8 +1994,7 @@ NPError NPP_NewStream(NPP instance,
      }
 
      /* Find a suitable command, fall back to savedMimeType if defined */
-     while(!find_command(instance, 1, stream->url) 
-                                 && !find_command(instance, 0, stream->url))
+     while(!find_command(THIS, 0, stream->url))
      {
           if(savedMimetype)       /* Lets try the saved one */
           {
@@ -1992,10 +2065,14 @@ void NPP_StreamAsFile(NPP instance,
 static void resize_window(NPP instance)
 {
      XSetWindowAttributes attrib;
+     data_t * THIS;
 
      /* Mozilla work around not require for versions > 0.13 */
      if((browserApiMajorVer > 0) || (browserApiMinorVer > 13))
+     {
          return;
+     }
+     THIS = instance->pdata;
 
      attrib.override_redirect = True;
      XChangeWindowAttributes(THIS->display, (Window)THIS->windata.window,
@@ -2028,6 +2105,7 @@ static void resize_window(NPP instance)
  ****************************************************************************/
 NPError NPP_SetWindow(NPP instance, NPWindow* window)
 {
+     data_t * THIS;
      D("SetWindow() - instance=%p\n", instance);
 
      if (!instance)
@@ -2047,7 +2125,9 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 
      if (!window->ws_info)
 	  return NPERR_NO_ERROR;
-     
+    
+     THIS = instance->pdata;
+
      THIS->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
      THIS->displayname = XDisplayName(DisplayString(THIS->display));
      THIS->windata = *window;
@@ -2058,7 +2138,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
           {
                /* Can only use streaming commands, as Mozilla cannot handle
                 * these types (mms) of urls */
-               if (!find_command(instance, 1, THIS->mms))
+               if (!find_command(THIS, 1, THIS->mms))
                {
 	           THIS->mms = NULL;
 	           NPN_Status(instance, 
@@ -2066,6 +2146,10 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
  	           return NPERR_GENERIC_ERROR;
                }
           }
+
+          /* Extract from the URL the various additional information */
+          parseURL(THIS, THIS->mms);
+
 	  new_child(instance, THIS->mms);
 	  THIS->mms = NULL;
 	  return NPERR_NO_ERROR;
