@@ -1,21 +1,3 @@
-#include <unistd.h>
-#include <ctype.h>
-#include <string.h>
-#include <stdio.h>
-#include <sysexits.h>
-#include <signal.h>
-#include <stdarg.h>
-#include <npapi.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <errno.h>
-#include <sys/time.h>
-#include <X11/X.h>
-#include <X11/Xos.h>
-#include <X11/Xutil.h>
-#include <X11/Xatom.h>
-#include <sys/socket.h>
-
 /*****************************************************************************
  * Defines
  *****************************************************************************/
@@ -66,12 +48,23 @@
 #define H_CONTROLS 	0x1000
 #define H_LINKS         0x2000
 #define H_FMATCH        0x4000
+#define H_AUTOSTART     0x8000
 
 /*****************************************************************************
  * mozplugger-common.c functions
  *****************************************************************************/
 #ifdef DEBUG
-void close_debug();
+
+#define DEBUG_FILENAME "mozdebug"
+void close_debug(void);
+char * get_debug_path(void);
+
 #endif
+
+#ifdef __GNUC__
+void D(char *fmt, ...) __attribute__((format(printf,1,2)));
+#else
 void D(char *fmt, ...);
+#endif
+
 void my_kill(pid_t pid);
